function bacteria(N,niter,delta,theta)
% function bacteria(N, niter, delta, theta)
%
% Implements Ermentrout and Edelstein-Keshet's bacterial growth cellular
% automaton model.  The initial condition is a single bacterial cell in an
% N-by-N two-dimensional space with sites that each contain a certain
% amount of food.  At each iteration, a certain amount of food is eaten by
% existing bacteria and even more is eaten by newly formed bacteria (within
% m=8 iterations).  Food also diffuses according to the diffusion facter,
% delta.  New growth occurs every m iterations with a probability of 0.5 in
% any sites in which the product of the food and a crowding function,
% k(number of nearby cells)=[0 40 40 40 30 20 10 0 0], is above a
% threshold, theta.
%
% Example usage: bacteria(201,500,0.25,250)
%
% (C) Dylan Hirsch-Shell, Nov 2006

n0 = 100; % initial food
a1 = 60; % food for growth
a2 = 10; % food for sustenance
m = 8; % growth every m time steps
k = [0 40 40 40 30 20 10 0 0]; % crowding function

cells = zeros(N); % NxN site array to begin with
new = cells;
n = cells + n0;

cells(round(N/2),round(N/2)) = 1;

for i = 1:niter
% Code for increasing the size of the CA as the colony grows:
%     [ypos,xpos] = find(cells);
%     if (min(xpos) <= 4)
%         cells = [zeros(size(cells,1),1) cells];
%         new = [zeros(size(new,1),1) new];
%         n = [n0*ones(size(n,1),1) n];
%     end
%     if (max(xpos) >= size(cells,2)-5)
%         cells = [cells zeros(size(cells,1),1)];
%         new = [new zeros(size(new,1),1)];
%         n = [n n0*ones(size(n,1),1)];
%     end
%     if (min(ypos) <= 4)
%         cells = [zeros(1,size(cells,2)); cells];
%         new = [zeros(1,size(new,2)); new];
%         n = [n0*ones(1,size(n,2)); n];
%     end
%     if (max(ypos) >= size(cells,1)-5)
%         cells = [cells; zeros(1,size(cells,2))];
%         new = [new; zeros(1,size(new,2))];
%         n = [n; n0*ones(1,size(n,2))];
%     end
% 
    [nrows,ncolumns]=size(n);
    average = n;
    for row = 2:(nrows-1)
        for column = 2:(ncolumns-1)
            N = n(row-1,column);
            S = n(row+1,column);
            E = n(row,column+1);
            W = n(row,column-1);
            NE = n(row-1,column+1);
            SE = n(row+1,column+1);
            SW = n(row+1,column-1);
            NW = n(row-1,column-1);
            average(row,column) = (4 * (N + S + E + W) + (NE + SE + SW + NW))/20;
        end
    end
    eaten = cells.*a2 + new.*a1;
    n = (1-delta)*n + delta * average - eaten;

    if mod(i,m)==0
        cells = cells + new;
        [nrows,ncolumns]=size(cells);
        Nneighbors = zeros(size(cells));
        for row = 2:(nrows-1)
            for column = 2:(ncolumns-1)
                Nneighbors(row,column) = cells(row-1,column)...
                    +cells(row+1,column)...
                    +cells(row,column+1)...
                    +cells(row,column-1)...
                    +cells(row-1,column+1)...
                    +cells(row+1,column+1)...
                    +cells(row+1,column-1)...
                    +cells(row-1,column-1);
            end
        end
        kn = k(Nneighbors + 1).*n;
        above_threshold = kn>theta;
        prob = rand(size(above_threshold)).*above_threshold;
        new = prob>0.5;
        new = (new - cells) > 0;
    end
end

[ypos,xpos]=find(cells);

figure; pcolor(n)
set(gca,'YDir','reverse','XTick',[],'YTick',[])
axis image
axis([min(xpos)-1 max(xpos)+2 min(ypos)-1 max(ypos)+2]);
colorbar
shading flat
figure;
pcolor(cells+new)
set(gca,'YDir','reverse','XTick',[],'YTick',[])
axis image
axis([min(xpos)-1 max(xpos)+2 min(ypos)-1 max(ypos)+2]);
map = [1 1 1;0 0 0];
caxis([0 1])
colormap(gca,map)
shading flat
